import warnings
from typing import List, Mapping, Optional

import arrow

from vital.api.api import API


class Sleep(API):
    """Endpoints for getting sleep data."""

    def get(
        self,
        user_id: str,
        start_date: str,
        end_date: Optional[str] = "",
        provider: Optional[str] = "",
    ) -> Mapping[str, List[Mapping]]:
        """
        GET Sleep data.
        """
        return self.client.get(
            f"/summary/sleep/{user_id}",
            params={
                "start_date": start_date,
                "end_date": end_date,
                "provider": provider,
            },
        )

    def get_stream_for_date_range(
        self,
        user_id: str,
        start_date: str,
        end_date: Optional[str] = "",
        provider: Optional[str] = "",
    ) -> Mapping[str, List[Mapping]]:
        """
        GET Sleep stream data for a single sleep event's within date range.
        """
        if not end_date:
            end = arrow.get()

        if (end - arrow.get(start_date)).days > 7:
            warnings.warn(
                "WARNING: calling get_stream_for_date_range for "
                "more than 7 days can significantly increase "
                "the latency of the request. "
                "Please consider reducing the number of days requested."
            )

        summaries = self.get(user_id, start_date, end_date, provider)
        for sleep in summaries["sleep"]:
            sleep["sleep_stream"] = self.get_stream(sleep["id"])

        return summaries

    def get_raw(
        self,
        user_id: str,
        start_date: str,
        end_date: Optional[str] = "",
        provider: Optional[str] = "",
    ) -> Mapping[str, List[Mapping]]:
        """
        GET Sleep data.
        """
        return self.client.get(
            f"/summary/sleep/{user_id}/raw",
            params={
                "start_date": start_date,
                "end_date": end_date,
                "provider": provider,
            },
        )

    def get_stream(self, sleep_id: str) -> Mapping[str, List[Mapping]]:
        """
        GET Sleep stream data for a single sleep event.
        """
        return self.client.get(f"/timeseries/sleep/{sleep_id}/stream")



from typing import Any, Dict, List, Mapping, Optional

from vital.api.api import API


class LabTests(API):
    """V3 endpoints for managing testkit orders."""

    def get_requisition_pdf(self, order_id: str) -> Any:
        """
        Gets the order requisition form in PDF format.
        """
        headers = {"Accept": "application/pdf"}
        return self.client.get(
            f"/order/{order_id}/requisition/pdf", api_version="v3", headers=headers
        )

    def register_testkit_order(
        self,
        user_id: str,
        sample_id: str,
        patient_details: dict,
        patient_address: dict,
        physician: Optional[dict] = None,
        consents: Optional[List[dict]] = None,
    ) -> Mapping[str, str]:
        """Register a testkit"""
        if not consents:
            consents = []
        params = {
            "user_id": user_id,
            "sample_id": sample_id,
            "patient_details": patient_details,
            "patient_address": patient_address,
            "consents": consents,
        }
        if physician:
            params["physician"] = physician

        return self.client.post("/order/testkit/register", params, api_version="v3")

    def create_unregistered_testkit_order(
        self,
        user_id: str,
        lab_test_id: str,
        shipping_details: dict,
    ) -> Mapping[str, str]:
        """Order an unregistered testkit"""
        params = {
            "user_id": user_id,
            "lab_test_id": lab_test_id,
            "shipping_details": shipping_details,
        }

        return self.client.post("/order/testkit", params, api_version="v3")

    def create_order(
        self,
        user_id: str,
        patient_details: Dict,
        patient_address: Dict,
        lab_test_id: str,
        physician: Optional[Dict] = None,
        health_insurance: Optional[Dict] = None,
        priority: bool = False,
        consents: Optional[List[dict]] = None,
    ) -> Mapping[str, str]:
        """Create new order"""
        if not consents:
            consents = []
        params = {
            "user_id": user_id,
            "patient_details": patient_details,
            "patient_address": patient_address,
            "lab_test_id": lab_test_id,
            "physician": physician,
            "priority": priority,
            "consents": consents,
        }
        if health_insurance:
            params["health_insurance"] = health_insurance

        return self.client.post("/order", params, api_version="v3")

    def create_test(
        self,
        name: str,
        description: str,
        markers: List[str],
        lab_id: int,
        method: str,
        sample_type: str,
    ) -> Mapping[str, str]:
        """Create new lab test"""
        params = {
            "name": name,
            "description": description,
            "lab_id": lab_id,
            "method": method,
            "sample_type": sample_type,
            "marker_ids": markers,
        }

        return self.client.post("/lab_tests", params, api_version="v3")

    def get_orders(self, page: int = 1, size: int = 50) -> Mapping[str, str]:
        """Get all orders"""
        return self.client.get(
            "/orders",
            {"page": page, "size": size},
            api_version="v3",
        )

    def get_order(self, order_id: str) -> Mapping[str, str]:
        """
        Get order status.
        :param str user_id: The order_id.
        """
        return self.client.get(f"/order/{order_id}", api_version="v3")

    def cancel_order(self, order_id: str) -> Mapping[str, str]:
        """
        Cancel order.
        :param str user_id: The order_id.
        """
        return self.client.post(f"/order/{order_id}/cancel", api_version="v3")

    def search_payor(
        self, insurance_name: str, insurance_state: Optional[str] = None
    ) -> Mapping[str, str]:
        params = {"insurance_name": insurance_name}

        if insurance_state:
            params["insurance_state"] = insurance_state

        return self.client.post("/insurance/search/payor", params, api_version="v3")

    def search_diagnosis(self, diagnosis_query: str) -> Mapping[str, str]:
        return self.client.get(
            f"/insurance/search/diagnosis?diagnosis_query={diagnosis_query}",
            api_version="v3",
        )

    def get_tests(self) -> Mapping[str, str]:
        """
        GET all lab tests the team has access to.
        """
        return self.client.get("/lab_tests", api_version="v3")

    def get_results(self, order_id: str) -> Mapping[str, str]:
        """
        GET both metadata and raw json test data.
        """
        return self.client.get(f"/order/{order_id}/result", api_version="v3")

    def get_results_pdf(self, order_id: str) -> Mapping[str, str]:
        """
        GET gets the lab result for the order in PDF format.
        """
        return self.client.get(f"/order/{order_id}/result/pdf", api_version="v3")

    def get_results_metadata(self, order_id: str) -> Mapping[str, str]:
        """
        GET metadata related to order results, such as
        lab metadata, provider and sample dates.
        """
        return self.client.get(f"/order/{order_id}/result/metadata", api_version="v3")

    def get_area_info(self, zip_code: str) -> Mapping[str, str]:
        """
        GET area info for a given zip code.
        """
        return self.client.get(
            "/order/area/info", {"zip_code": zip_code}, api_version="v3"
        )



import enum
import typing as t
from datetime import date, datetime

import pydantic as pyd

Longitude = pyd.confloat(ge=-180, le=180)
Latitude = pyd.confloat(ge=-90, le=90)


class LngLat(pyd.BaseModel):
    lng: Longitude
    lat: Latitude


class USStateCode(pyd.ConstrainedStr):
    regex = r"^[A-Z]{2}$"


class USZipCode(pyd.ConstrainedStr):
    regex = r"^\d{5}$"


class USAddress(pyd.BaseModel):
    first_line: pyd.StrictStr
    second_line: t.Optional[pyd.StrictStr]
    city: pyd.StrictStr
    state: USStateCode
    zip_code: USZipCode
    unit: t.Optional[pyd.StrictStr]


class AppointmentType(str, enum.Enum):
    PHLEBOTOMY = "phlebotomy"


class AppointmentProvider(str, enum.Enum):
    GETLABS = "getlabs"


class AppointmentStatus(str, enum.Enum):
    CONFIRMED = "confirmed"
    PENDING = "pending"
    IN_PROGRESS = "in_progress"
    COMPLETED = "completed"
    CANCELLED = "cancelled"


class Appointment(pyd.BaseModel):
    id: pyd.UUID4
    user_id: pyd.UUID4
    address: USAddress
    location: LngLat
    start_at: datetime
    end_at: datetime
    iana_timezone: str
    type: AppointmentType
    provider: AppointmentProvider
    status: AppointmentStatus
    provider_id: str
    can_reschedule: bool


class AppointmentSlot(pyd.BaseModel):
    booking_key: str
    start: datetime
    end: datetime
    expires_at: t.Optional[datetime]
    price: float
    is_priority: bool
    num_appointments_available: float


class DaySlot(pyd.BaseModel):
    date: date
    slots: t.List[AppointmentSlot]


class AppointmentAvailability(pyd.BaseModel):
    timezone: str
    slots: t.List[DaySlot]


class CancellationReason(pyd.BaseModel):
    id: pyd.UUID4
    name: str
    is_refundable: bool



import typing as t
import uuid

import pydantic as pyd

from vital.api.api import API
from vital.api.schema.athome_phlebotomy import (
    Appointment,
    AppointmentAvailability,
    CancellationReason,
    USAddress,
)


class AtHomePhlebotomy(API):
    """Endpoints for managing at-home phlebotomy appointments."""

    def appointment_availability(
        self,
        order_id: uuid.UUID,
        address: t.Optional[t.Union[dict, USAddress]] = None,
    ) -> AppointmentAvailability:
        params = None
        if address is not None:
            if isinstance(address, dict):
                params = address

            elif isinstance(address, USAddress):
                params = address.dict()

            else:
                raise TypeError(
                    f"address must be a dict or USAddress, not {type(address)}"
                )

        response_body = self.client.post(
            f"/order/{order_id}/phlebotomy/appointment/availability",
            params,
            api_version="v3",
        )

        return AppointmentAvailability.parse_obj(response_body)

    def book_appointment(
        self,
        order_id: uuid.UUID,
        booking_key: str,
    ) -> Appointment:
        response_body = self.client.post(
            f"/order/{order_id}/phlebotomy/appointment/book",
            {
                "booking_key": booking_key,
            },
            api_version="v3",
        )

        return Appointment.parse_obj(response_body)

    def reschedule_appointment(
        self,
        order_id: uuid.UUID,
        booking_key: str,
    ) -> Appointment:
        response_body = self.client.patch(
            f"/order/{order_id}/phlebotomy/appointment/reschedule",
            {
                "booking_key": booking_key,
            },
            api_version="v3",
        )

        return Appointment.parse_obj(response_body)

    def cancel_appointment(
        self, order_id: uuid.UUID, cancellation_reason_id: uuid.UUID
    ) -> Appointment:
        params = {
            "cancellation_reason_id": cancellation_reason_id,
        }
        response_body = self.client.patch(
            f"/order/{order_id}/phlebotomy/appointment/cancel",
            params,
            api_version="v3",
        )

        return Appointment.parse_obj(response_body)

    def cancellation_reasons(self) -> t.List[CancellationReason]:
        response_body = self.client.get(
            "/order/phlebotomy/appointment/cancellation-reasons",
            api_version="v3",
        )

        return pyd.parse_obj_as(
            t.List[CancellationReason],
            response_body,
        )

    def get_appointment(self, order_id: uuid.UUID) -> Appointment:
        response_body = self.client.get(
            f"/order/{order_id}/phlebotomy/appointment",
            api_version="v3",
        )

        return Appointment.parse_obj(response_body)
